// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: CountsSpectrum
// Author: Hassan Siddiqui
//
// Description: 
// Provides an clean interface to an OGIP PHA spectrum.
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: CountsSpectrum.h,v 1.4 1999/05/25 13:15:51 hsiddiqu Exp $

#ifndef CountsSpectrum_H
#define CountsSpectrum_H

// General classes

#include <string>
#include <iostream.h>
#include <vector>
#include <map.h>

// SAS-specific classes

#include "Dal.h"
#include "errstr.h"

// Utility functions
#include "Tracer.h"

// -----------------------------------------------------------------------------------

// class CountsSpectrum

// Provides an clean interface to an OGIP PHA spectrum.

// -----------------------------------------------------------------------------------

typedef map<unsigned int, double, less<double> > SpectrumMap;

class CountsSpectrum
{

public:

  /* Constructor */

  CountsSpectrum(const string & spectrumName):
    _rows(0),
    _normalization(0)
    {
      PROC;

      // Read file
      SAS_DEBUG (cout << "opening dataset "<< spectrumName << ".." <<endl;);
      DataSet * set   = dataSetServer->open(spectrumName, DataSet::Read);
     
      // Open the spectrum table
      Table * table = set->table("SPECTRUM");

      // Check if COUNTS column exists. If not, look at RATE column.
      bool unitsAreCounts;
      Column * countsColumn;

      if (table->hasColumn("COUNTS"))
	{
	  unitsAreCounts = true;
	  countsColumn =  table->column("COUNTS");
	}
      else if (table -> hasColumn("RATE"))
	{
	  unitsAreCounts = false;
	  countsColumn = table->column("RATE");
	}
      else
	{
	  // ERROR
	  Sas::error("noCountOrRateColumn","There is neither a count or rate column in dataset");
	}
      ColumnData * countsData  = countsColumn->data();


      // Get channel column

      Column * channelColumn = table->column("CHANNEL");
      
      ColumnData * channelData = channelColumn->data(); 



      // Generate map of spectrum (or use the histogram library?)

      _normalization = 0;
      double countsValue;

      for (unsigned int i = 0; i < countsColumn->rows(); ++i)
	{
	  if (unitsAreCounts)
	    {
	      countsValue = countsData->int32Data()[i];
	    }
	  else
	    {
	      countsValue = countsData->real32Data()[i];
	    }
	  
	  // Populate map, taking into account that channel numbers start
	  // from 1.
	  _spectrum[ channelData->int16Data()[i] - 1 ] = countsValue;

	  _normalization += countsValue;

	}

      SAS_DEBUG( cout << " CountsSpectrum:: normalization " 
	     << _normalization << endl;);

      // Keep track of how many rows there are in the origin
      _rows =  countsColumn->rows();

      // Get keys to first and last entries to map. 
      // These can be used to recover in the case that data is requested
      // that is out of bounds

      _firstChannel = (*_spectrum.begin()).first;
      _lastChannel = (*_spectrum.rbegin()).first;

      SAS_DEBUG( cout << "Assigning first channel " << _firstChannel << endl
	     << " and last channel " << _lastChannel << endl;);

      // Clean up DAL pointers and close dataset

      delete channelData;
      delete countsData;

      SAS_DEBUG( cout << "closing dataset " << spectrumName << ".." <<endl;);
      dataSetServer->close(set);
 
      ENDPROC;

    }

  const unsigned int firstChannel() const
    {
      PROC; RETURN _firstChannel;
    }

  const unsigned int lastChannel() const
    {
      PROC; RETURN _lastChannel;
    }


  const double countrate(const unsigned int channel) const
    {
      PROC;

      // Check if channel is within range defined by spectrum

      if ( channel < _firstChannel || channel > _lastChannel )
	{
	  errstr << " Channel " << channel << " is out of range " 
		 << Sas::error("channelOutOfRange") << endl;
	};

      assert ( channel >= _firstChannel && channel <= _lastChannel );

      assert(_spectrum.count(channel) > 0);


      // Get countrate.
      // Note that could simply used _spectrum[ channel ] here but
      // this is not a constant method; moreover, it ADDS data to the _spectrum
      // map is channel is not valid. It may be nicer to write a generic
      // wrapper to the Map class which has an bool exists() method.
      
      SpectrumMap::const_iterator itr = _spectrum.find(channel);

      assert (itr != _spectrum.end() );

      RETURN (*itr).second;
    }

  unsigned int rows() { PROC; RETURN _rows; }

private:

  // Suppress default constructor
  CountsSpectrum();

//   const int _LOWER_BOUND_VALUE = 0;
//   const int _UPPER_BOUND_VALUE = 0;

  unsigned int _rows;

  double _normalization;

  unsigned int _firstChannel;
  unsigned int _lastChannel;

  SpectrumMap _spectrum;
};


#endif




