// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: GaussianResponseVector
// Author: Hassan Siddiqui
//
// Description: Produces a fragmented row response. 
// For testing the grouping algorithm in ResponseGrouping.h
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: GaussianResponseVector.h,v 1.1 2000/03/28 12:42:49 hsiddiqu Exp $

#ifndef GaussianResponseVector_H
#define GaussianResponseVector_H

// General classes

#include <iostream.h>
#include <vector>

// SAS-specific classes

#include "Dal.h"

// Utility functions
#include "Tracer.h"

// Local functions 
#include "GenericResponseVector.h"

// -----------------------------------------------------------------------------------

// class GaussianResponseVector

// Returns a simple gaussian response. 

// -----------------------------------------------------------------------------------

class GaussianResponseVector : public GenericResponseVector
{

public:

  GaussianResponseVector(unsigned int vectorLength, double mean, double sigma ):
    _vectorLength(vectorLength),
    _mean(mean),
    _sigma(sigma)
    {
      PROC;
      ENDPROC
    }

  const vector<double> response() const
    {
      PROC;
      
      vector<double> response;

      for (unsigned int i = 0; i < _vectorLength; ++i)
	{
	  // Compute exponent term, taking care of unsigned int conversions
	  double exp_term = (double(i) - double(_mean)) / _sigma;
	  double value = exp( (-0.5) * exp_term * exp_term );
	  
	  DEBUG (cout << "GaussianResponseVector: position " << i
		 << " mean " << _mean
		 << " exp_term " << exp_term
		 << " value " << value 
		 << endl;);
 
	  response.push_back( value ); 
	}

      RETURN response;
    }


private:
  
  // Suppress default constructor
  GaussianResponseVector();

  unsigned int _vectorLength;
  double _mean;
  double _sigma;

};


#endif




