// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: InterpolatedData
// Author: Hassan Siddiqui
//
// Modifications: See ChangeLog
// 
// $Id: InterpolatedData.h,v 1.2 2000/03/28 12:42:50 hsiddiqu Exp $

#ifndef InterpolatedData_H
#define InterpolatedData_H

// General classes

#include <iostream.h>
#include <math.h>
#include <map.h>

// Utility functions
#include "Tracer.h"

// Local functions
#include "Interpolator.h"

// =================================================================

// class InterpolatedData

// Stores data in the form of key=double, value=<class T>,
// and provides a method to interpolate/extrapolate that data.
// Need to ensure that there are at least two elements in map. 
// A method, check(), is provided to make this check.
// Note that class T can only correspond to those classes where
// Interpolator::calc(double, double, double, T, T) is defined.

// =================================================================

template <class T>
class InterpolatedData
{

public:

  InterpolatedData( unsigned int elements = 1)
    : _elements(elements){};

  void add ( double key, T data)
    {
      PROC;

      /* Worry about checking the no. elements later - a bit tricky when
       * dealing with primitive types such as doubles */
//       assert (data.size() == _elements);

      _map[key] = data;

       /* Store last and last-but-one iterator 
	* to speed up execution of value() */
      _itrLast          = _map.end();
      _itrLastButOne    = --_itrLast;
      _itrLastButOne--;

      ENDPROC;
   }

  bool check() const
    {
      PROC;
      RETURN (_map.size() > 1);
    }

  double size() const
    {
      PROC;
      RETURN _map.size();
    }

  T value ( double key ) const
    {
      PROC;
      assert(_map.size() > 1);

      map<double, T, less<double> >::const_iterator itr_hi = _map.lower_bound(key);
      map<double, T, less<double> >::const_iterator itr_lo;

      if (itr_hi == _map.end())
	{
	  itr_lo = _itrLastButOne;
	  itr_hi = _itrLast;
	}
      else if (itr_hi == _map.begin())
	{
	  itr_lo = itr_hi;
	  itr_hi++;
	}
      else
	{
	  itr_lo = itr_hi;
	  itr_lo--;
	}

      RETURN Interpolator::calc( key, 
				 (*itr_lo).first, 
				 (*itr_hi).first,
				 (*itr_lo).second,
				 (*itr_hi).second);
   
    };

private:

  map<double, T, less<double> > _map;
  map<double, T, less<double> >::const_iterator _itrLast;
  map<double, T, less<double> >::const_iterator _itrLastButOne;

  unsigned int _elements;
  
};
#endif




