// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: MOSPatternFuncs
// Author: Hassan Siddiqui
//
// Modifications: See ChangeLog
// 
// $Id: MOSPatternFuncs.h,v 1.1 1999/02/04 16:13:43 hsiddiqu Exp $

#ifndef MOSPatternFuncs_H
#define MOSPatternFuncs_H

// SAS-specific classes
#include "Dal.h"

// General classes
#include <math.h>

// General utility functions
#include "Tracer.h"

// Local classes
#include "SplitPatternFuncs.h"
#include "SplitEventsData.h"


// ===========================================================================

// class ProbabilityFunctions

// Namespace for common functions used in various PatternFunc implementations

// ===========================================================================

class ProbabilityFunctions
{
public:
  static double prob1 (const SplitEventsDist & alpha, double flux)
    {
      PROC;
      RETURN 1-exp(-alpha[0] * flux);
    }

  static double prob2 (const SplitEventsDist & alpha, double flux)
    {
      PROC;
      RETURN 1-exp(-alpha[1] * flux * 0.5);
    }
};

// ===========================================================================

// class MOSMonoPixelPatternFuncs

// Contains algorithms to correct for pile-up where the pattern of interest
// is of mono-pixel type.

// ===========================================================================

class MOSMonoPixelPatternFuncs : public SplitPatternFuncs
{
public:
  
  /* Constructors */
  MOSMonoPixelPatternFuncs(const SplitEventsData & splitEventsData) :
    SplitPatternFuncs(splitEventsData){};

  ~MOSMonoPixelPatternFuncs(){};

protected:

  /* Note flux is in units of /frame/pixel */

  const PileupData getPileup( const SplitEventsDist& alpha,
			       double flux )  const
    {
      PROC;

      double alpha_flux     = alpha[0] * flux;

      double gamma = 9 + (3 * alpha[1]) + (6 * alpha[2]) + (7 * alpha[3]);

      double exp_gamma_flux = exp(-gamma*flux);

      double mu             = ( exp(alpha_flux) - 1) * exp_gamma_flux;

      double mu_t           =  alpha_flux * exp_gamma_flux;


//       cout << " flux "    << flux    << endl
// 	   << " alpha[0] " << alpha[0] << endl
// 	   << " mu "      << mu      << endl
// 	   << " mu_t "    << mu_t    << endl;

      RETURN PileupData(mu, mu_t);
    }

};


// ===========================================================================

// class MOSBiPixelPatternFuncs

// Contains algorithms to correct for pile-up where the pattern of interest
// is of Bi-pixel type.

// ===========================================================================

class MOSBiPixelPatternFuncs : public SplitPatternFuncs
{
public:
  
  /* Constructors */
  MOSBiPixelPatternFuncs(const SplitEventsData & splitEventsData) :
    SplitPatternFuncs(splitEventsData){};

  ~MOSBiPixelPatternFuncs(){};

protected:

  /* Note flux is in units of /frame/pixel */

  const PileupData getPileup( const SplitEventsDist& alpha,
			       double flux )  const
    {
      PROC;

      double gamma = 12 + (3.5 * alpha[1]) + (7 * alpha[2]) + (8 * alpha[3]);

      double p1 = ProbabilityFunctions::prob1(alpha, flux);
      double p2 = ProbabilityFunctions::prob2(alpha, flux);

      double mu = 2*(p2 + (1-p2)*(p1*p1)) * 
	exp(-(gamma -2*alpha[0] - 0.5*alpha[1])*flux);

      double mu_t  =  alpha[0] * flux * exp(-gamma*flux);


//       cout << " flux "     << flux     << endl
// 	   << " alpha[0] " << alpha[0] << endl
// 	   << " alpha[1] " << alpha[1] << endl
// 	   << " mu "       << mu       << endl
// 	   << " mu_t "     << mu_t     << endl;

      RETURN PileupData(mu, mu_t);
    }

};

// ===========================================================================

// class MOSTriPixelPatternFuncs

// Contains algorithms to correct for pile-up where the pattern of interest
// is of Tri-pixel type.

// ===========================================================================

class MOSTriPixelPatternFuncs : public SplitPatternFuncs
{
public:
  
  /* Constructors */
  MOSTriPixelPatternFuncs(const SplitEventsData & splitEventsData) :
    SplitPatternFuncs(splitEventsData){};

  ~MOSTriPixelPatternFuncs(){};

protected:

  /* Note flux is in units of /frame/pixel */

  const PileupData getPileup( const SplitEventsDist& alpha,
			       double flux )  const
    {
      PROC;

      double gamma = 15 + (4 * alpha[1]) + (7.75 * alpha[2]) + (9 * alpha[3]);

      double p1 = ProbabilityFunctions::prob1(alpha, flux);
      double p2 = ProbabilityFunctions::prob2(alpha, flux);

      double t3 = 
	( p1*p1*p1/3. * exp(-alpha[1]*flux )) +
	( p1*p2/2.    * exp(-alpha[1]*flux/2. )) +
	( 0.75*p2*p2 );

      double mu = 
	4 * ( 1 - (exp(-0.25*alpha[2]*flux) * (1 - t3))) *
	( exp(-(gamma - 3*alpha[0] - alpha[1] - 0.25*alpha[2])*flux) ); 

      double mu_t  =  alpha[2] * flux * exp(-gamma*flux);


//       cout << " flux "     << flux     << endl
// 	   << " alpha[0] " << alpha[0] << endl
// 	   << " alpha[1] " << alpha[1] << endl
// 	   << " alpha[2] " << alpha[2] << endl
// 	   << " mu "       << mu       << endl
// 	   << " mu_t "     << mu_t     << endl;

      RETURN PileupData(mu, mu_t);
    }

};

// ===========================================================================

// class MOSQuadPixelPatternFuncs

// Contains algorithms to correct for pile-up where the pattern of interest
// is of Quad-pixel type.

// ===========================================================================

class MOSQuadPixelPatternFuncs : public SplitPatternFuncs
{
public:
  
  /* Constructors */
  MOSQuadPixelPatternFuncs(const SplitEventsData & splitEventsData) :
    SplitPatternFuncs(splitEventsData){};

  ~MOSQuadPixelPatternFuncs(){};

protected:

  /* Note flux is in units of /frame/pixel */

  const PileupData getPileup( const SplitEventsDist& alpha,
			       double flux )  const
    {
      PROC;

      double gamma = 16 + (4 * alpha[1]) + (8 * alpha[2]) + (9 * alpha[3]);

      double p1 = ProbabilityFunctions::prob1(alpha, flux);
      double p2 = ProbabilityFunctions::prob2(alpha, flux);

      double p2sqd = p2*p2;

      double q2 = 
	( p1*p1*p1*p1 * exp(-2.  * alpha[1] * flux) ) +
	( 4*p1*p1*p2  * exp(-1.5 * alpha[2] * flux) ) +
	( 4*p1*p2sqd  * exp(-alpha[1]*flux) ) +
	( p2sqd*(1-p2sqd) );   

      double q3 =
	1 - ( exp(-alpha[2]*flux) * (1-q2)) -
	(4*(1-exp(-0.25*alpha[2]*flux))*
	exp(-(alpha[0] + alpha[1] + 0.75*alpha[2])*flux));
      
      double mu = 
	(1 - (exp(-alpha[3]*flux)*(1-q3))) *
	( exp(-(gamma - 1 - 3*alpha[0] - alpha[1])*flux)) ; 

      double mu_t  =  alpha[3] * flux * exp(-gamma*flux);


//       cout << " flux "     << flux     << endl
// 	   << " alpha[0] " << alpha[0] << endl
// 	   << " alpha[1] " << alpha[1] << endl
// 	   << " alpha[2] " << alpha[2] << endl
// 	   << " alpha[3] " << alpha[3] << endl
// 	   << " mu "       << mu       << endl
// 	   << " mu_t "     << mu_t     << endl;

      RETURN PileupData(mu, mu_t);
    }

};
#endif
