// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: TablePhotonDistribution
// Author: Hassan Siddiqui
//
// Modifications: See ChangeLog
// 
// $Id: TablePhotonDistribution.h,v 1.3 1999/05/25 13:15:53 hsiddiqu Exp $

#ifndef TablePhotonDistribution_H
#define TablePhotonDistribution_H

// General classes

#include <string>
#include <iostream.h>
#include <vector>
#include <math.h>

// SAS-specific classes

#include "Dal.h"
#include "errstr.h"

// Utility functions
#include "Tracer.h"

// Local functions
#include "PhotonDistribution.h"
#include "InterpolatedData.h"

// ========================================================================

// class TablePhotonDistribution

// Uses a model of the photon distribution stored in a DAL dataset.

// ========================================================================

class TablePhotonDistribution : public PhotonDistribution
{

public:

  /* Constructor */

  /* NB scaleFactor is a conversion factor to convert
   * input data units to output flux units (eg ergs/s/cm2/keV -> photons/s)
   * Will tidy flux and rate definitions later.
   */

  TablePhotonDistribution( const string& dataSetName,
			   double        scaleFactor = 1) :
    _scaleFactor(scaleFactor)
    {
      /* Open dataset */
      DataSet * set = dataSetServer->open(dataSetName, DataSet::Read);

      /* Access spectrum table */
      Table * table = set->table("DISTRIBUTION");

      /* Get energy column */
      Column * energyCol         = table->column("ENERGY");
      ColumnData * energyColData = energyCol->data();
      assert(energyColData->dataType() == Dal::Real32);
      real32 * energyData        = energyColData->real32Data();
      
             
      /* Get flux column */
      Column * fluxCol         = table->column("FLUX");
      ColumnData * fluxColData = fluxCol->data();
      assert(fluxColData->dataType() == Dal::Real32);
      real32 * fluxData        = fluxColData->real32Data();

      /* Populate map */
      for (unsigned int i = 0; i < table->rows(); ++i)
	{
	  _data.add(energyData[i], fluxData[i]);
	}

      delete fluxColData; fluxColData = 0;
      delete energyColData; energyColData = 0;
  
      dataSetServer->close(set);

      if (!_data.check())
	{
	  Sas::error("tooFewDataPoints",
		"Not enough data points in dataset - need at least two.");
	}
    }

  /* Determine flux (photons/s) for given energy */
  double flux( double energy ) const
    {
      PROC;

      double result = _data.value(energy) * _scaleFactor;

//       cout << " TablePhotonDistribution::flux() "
// 	   << " energy " << energy
// 	   << " result " << result
// 	   << endl;

      RETURN result;
    }

  /* Destructor */
  ~TablePhotonDistribution(){};

private:

  /* Suppress default constructor */
  TablePhotonDistribution();

  InterpolatedData<double> _data;

  double _scaleFactor;

};
#endif




