// Module: VectorRebinner.h
// Author: Hassan Siddiqui
//
// Description: Rebins an input vector.
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: VectorRebinner.cc,v 1.1 2000/03/28 12:43:00 hsiddiqu Exp $

/* General classes */

/* SAS-specific classes */
#include "errstr.h"
#include "Tracer.h"

/* Local classes */
#include "VectorRebinner.h"

/***********************************************************
 *
 * class VectorRebinner
 *
 ***********************************************************/

/* Constructor */
VectorRebinner::VectorRebinner()
{
  PROC;
  ENDPROC;
}

/* Return a vector of rebinned values given an input vector. */
const vector<double> VectorRebinner::rebin( const vector<double> & vect_in,
					    unsigned binfactor,
					    unsigned offset)
{
      PROC;

      if (binfactor == 1)
	{
	  ALTRETURNV(vect_in);
	}
      
      unsigned npixels = vect_in.size();
      errstr << "VectorRebinner::rebin: rebinning vector of "
	     << npixels << " elements "
	     << Sas::message(Msg::App, Msg::Verbose);

      vector<double> out;

      unsigned index = offset;
      while (index < npixels)
	{
	  double sum = 0;
	  for ( unsigned i = index; 
		(i < index + binfactor) && (i < npixels); 
		++i)
	    {
	      sum+= vect_in[i];
	    }
	  index+=binfactor;
	  out.push_back(sum);
	}
      
      /* Check size of output vector */
      unsigned expectedsize = (npixels-offset)/binfactor;
      expectedsize = 
	((expectedsize*binfactor) == npixels-offset) 
	? expectedsize : expectedsize+1;

      if (out.size() != expectedsize)
	{
	  errstr << "Number of bins in output vector ["<< out.size()
		 << "] does not match the number expected ["<< expectedsize
		 << "]" 
		 << Sas::error("VectorRebinnerInvalidOutputVector"); 
	}

      RETURN out;
}




